// this will take the file as a parameter and the output of get_header.sh as stdin.
// it will output on stdout:
// 	baseline
// 	baseline noise
// 	maximum
// 	time
// 	slope

#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>

using namespace std;

#define BEFORE_TRIGGER 50 // how many points before the trigger to stop counting the baseline
#define AFTER_TRIGGER 200 // how many points after the trigger to stop looking for the pulse

int main(int argc, char *argv[]){
	if(argc<2){
		cout << "need file name" << endl;
		return -1;
	}
	
	int num_points, trigger, num_records;
	float interval;
	
	cin >> num_points >> interval >> trigger >> num_records;
	
	ifstream in(argv[1]);
	
	cout << "#base	noise	max	time	slope" << endl;
	
	for(int pulse=0; pulse<num_records; pulse++){
		float base_tot=0, base_tot2=0, max=0, voltage[trigger+AFTER_TRIGGER], temp;
		int max_time;
		
		for(int point=0; point<num_points; point++){
			for(int i=0; i<4; i++){
				char comma=' ';
				while(comma!=',') in >> comma;
			}
			in >> temp;
			
			if(point>trigger+AFTER_TRIGGER)
				continue;
			
			voltage[point]=temp;
			
			if(voltage[point]<max){
				max=voltage[point];
				max_time=point;
			}
			
			if(point<trigger-BEFORE_TRIGGER){
				base_tot+=voltage[point];
				base_tot2+=pow(voltage[point],2);
			}
		}
		
		float pulse_time=-2, time_25=-2, time_75=-2, slope;
		
		ofstream tmp("tmp");
		for(int point=0; point<max_time; point++){
			tmp << voltage[point] << endl;
			if(voltage[point]<max/2 && pulse_time<-1)
				pulse_time=point;
			if(voltage[point]<max*0.3 && time_25<-1)
				time_25=point;
			if(voltage[point]<max*0.7 && time_75<-1)
				time_75=point;
		}
		slope=-2/(time_75-time_25);
		
		for(int point=0; point<20; point++) tmp << max << endl;
		tmp.close();
		
		ofstream gpfile("gp");
		gpfile << "f(x)=" << base_tot/(trigger-BEFORE_TRIGGER) << "+a*exp(b*(x-c))/(1+exp(b*(x-c)))" << endl;
		gpfile << "a=" << max << endl;
		gpfile << "b=" << slope << endl;
		gpfile << "c=" << pulse_time << endl;
		gpfile << "fit f(x) \"tmp\" via b,c" << endl;
		gpfile.close();
		
		system("gnuplot gp 2>tmp2");
		system("awk '$1==\"Final\" {here=1} \n\
		$1==\"b\" && here==1 {s=$3; ss=$5}\n\
		$1==\"c\" && here==1 {print s \" \" ss \" \" $3 \" \" $5; nextfile}' tmp2 > tmp3");
		ifstream tmp3("tmp3");
		
		tmp3 >> slope >> temp >> pulse_time >> temp;
		tmp3.close();
		cout << base_tot/(trigger-BEFORE_TRIGGER) << "\t" << sqrt(base_tot2/(trigger-BEFORE_TRIGGER)-pow(base_tot/(trigger-BEFORE_TRIGGER),2)) << "\t" << max << "\t" << pulse_time*interval << "\t" << slope/interval << endl;
		}
	return 0;
}
